@@ -1,16 +1,7 @@
-#include <sys/types.h>
-#include <sys/stat.h>
-
-#include <fcntl.h>
-#include <math.h>
-#include <stdio.h>
-#include <string.h>
-#include <unistd.h>
+#include "btpd.h"
 
 #include <openssl/sha.h>
-
-#include "btpd.h"
-#include "stream.h"
+#include <stream.h>
 
 struct content {
     enum { CM_INACTIVE, CM_STARTING, CM_ACTIVE } state;
@@ -62,7 +53,7 @@ BTPDQ_HEAD(std_tq, start_test_data);
 
 static struct std_tq m_startq = BTPDQ_HEAD_INITIALIZER(m_startq);
 
-static struct event m_workev;
+static struct timeout m_workev;
 
 #define READBUFLEN (1 << 14)
 
@@ -465,7 +456,7 @@ startup_test_run(void)
     if (std->start >= tp->npieces)
         startup_test_end(tp, 1);
     if (!BTPDQ_EMPTY(&m_startq))
-        event_add(&m_workev, (& (struct timeval) { 0, 0 }));
+        btpd_timer_add(&m_workev, (& (struct timespec) { 0, 0 }));
 }
 
 void
@@ -482,7 +473,7 @@ startup_test_begin(struct torrent *tp, struct file_time_size *fts)
         std->fts = fts;
         BTPDQ_INSERT_TAIL(&m_startq, std, entry);
         if (std == BTPDQ_FIRST(&m_startq))
-            event_add(&m_workev, (& (struct timeval) { 0, 0 }));
+            btpd_timer_add(&m_workev, (& (struct timespec) { 0, 0 }));
     } else {
         free(fts);
         startup_test_end(tp, 0);
@@ -548,5 +539,5 @@ cm_start(struct torrent *tp, int force_test)
 void
 cm_init(void)
 {
-    evtimer_set(&m_workev, worker_cb, NULL);
+    evtimer_init(&m_workev, worker_cb, NULL);
 }
