@@ -1,79 +1,96 @@
-#include <sys/types.h>
-#include <sys/file.h>
-#include <sys/stat.h>
-#include <arpa/inet.h>
-
-#include <err.h>
-#include <errno.h>
-#include <event.h>
-#include <evdns.h>
-#include <fcntl.h>
-#include <getopt.h>
-#include <stdio.h>
-#include <string.h>
-#include <unistd.h>
-
 #include "btpd.h"
 
-static void
-writepid(int pidfd)
+#include <getopt.h>
+#include <time.h>
+
+int btpd_daemon_phase = 2;
+int first_btpd_comm[2];
+int pidfd;
+
+void
+first_btpd_exit(char code)
 {
-    FILE *fp = fdopen(dup(pidfd), "w");
-    fprintf(fp, "%d", getpid());
-    fclose(fp);
+    write(first_btpd_comm[1], &code, 1);
+    close(first_btpd_comm[0]);
+    close(first_btpd_comm[1]);
 }
 
 static void
-setup_daemon(int daemonize, const char *dir, const char *log)
+writepid(void)
 {
-    int pidfd;
+    int nw;
+    char pidtxt[100];
+    nw = snprintf(pidtxt, sizeof(pidtxt), "%ld", (long)getpid());
+    ftruncate(pidfd, 0);
+    write(pidfd, pidtxt, nw);
+}
 
-    if (log == NULL)
-        log = "log";
+static void
+setup_daemon(int daemonize, const char *dir)
+{
+    char c;
+    pid_t pid;
+    struct timespec ts;
+
+    if (snprintf(NULL, 0, "btpd") != 4)
+        btpd_err("snprintf doesn't work.\n");
+
+    if (evtimer_gettime(&ts) != 0)
+        btpd_err("evtimer_gettime failed (%s).\n", strerror(errno));
 
     if (dir == NULL) {
         if ((dir = find_btpd_dir()) == NULL)
-            errx(1, "Cannot find the btpd directory");
-        else if (dir[0] != '/')
-            errx(1, "got non absolute path '%s' from system environment.",
+            btpd_err("Cannot find the btpd directory.\n");
+        if (dir[0] != '/')
+            btpd_err("Got non absolute path '%s' from system environment.\n",
                 dir);
         btpd_dir = dir;
     }
 
     if (mkdir(dir, 0777) == -1 && errno != EEXIST)
-        err(1, "Couldn't create home '%s'", dir);
+        btpd_err("Couldn't create home '%s' (%s).\n", dir, strerror(errno));
 
     if (chdir(dir) != 0)
-        err(1, "Couldn't change working directory to '%s'", dir);
+        btpd_err("Couldn't change working directory to '%s' (%s).\n", dir,
+            strerror(errno));
 
     if (mkdir("torrents", 0777) == -1 && errno != EEXIST)
-        err(1, "Couldn't create torrents subdir");
-
-    if ((pidfd = open("pid", O_CREAT|O_TRUNC|O_WRONLY, 0666)) == -1)
-        err(1, "Couldn't open 'pid'");
-
-    if (flock(pidfd, LOCK_NB|LOCK_EX) == -1)
-        errx(1, "Another instance of btpd is probably running in %s.", dir);
+        btpd_err("Couldn't create torrents subdir (%s).\n", strerror(errno));
 
     if (btpd_dir == NULL) {
         char wd[PATH_MAX];
         if (getcwd(wd, PATH_MAX) == NULL)
-            err(1, "couldn't get working directory");
-        btpd_dir = strdup(wd);
+            btpd_err("Couldn't get working directory (%s).\n",
+                strerror(errno));
+        if ((btpd_dir = strdup(wd)) == NULL)
+            btpd_err("Out of memory.\n");
     }
 
     if (daemonize) {
-        if (daemon(1, 1) != 0)
-            err(1, "Failed to daemonize");
-        freopen("/dev/null", "r", stdin);
-        if (freopen(log, "a", stdout) == NULL)
-            err(1, "Couldn't open '%s'", log);
-        dup2(fileno(stdout), fileno(stderr));
-        setlinebuf(stdout);
-        setlinebuf(stderr);
+        if (pipe(first_btpd_comm) < 0)
+            btpd_err("Failed to create pipe (%s).\n", strerror(errno));
+        if ((pid = fork()) < 0)
+            btpd_err("fork() failed (%s).\n", strerror(errno));
+        if (pid != 0) {
+            read(first_btpd_comm[0], &c, 1);
+            exit(c);
+        }
+        btpd_daemon_phase--;
+        if (setsid() < 0)
+            btpd_err("setsid() failed (%s).\n", strerror(errno));
+        if ((pid = fork()) < 0)
+            btpd_err("fork() failed (%s).\n", strerror(errno));
+        if (pid != 0)
+            exit(0);
     }
 
-    writepid(pidfd);
+    if ((pidfd = open("pid", O_CREAT|O_WRONLY, 0666)) == -1)
+        btpd_err("Couldn't open 'pid' (%s).\n", strerror(errno));
+
+    if (lockf(pidfd, F_TLOCK, 0) == -1)
+        btpd_err("Another instance of btpd is probably running in %s.\n", dir);
+
+    writepid();
 }
 
 static void
@@ -85,6 +102,13 @@ usage(void)
         "Usage: btpd [-d dir] [-p port] [more options...]\n"
         "\n"
         "Options:\n"
+        "-4\n"
+        "\tUse IPv4. If given in conjunction with -6, "
+        "both versions are used.\n"
+        "\n"
+        "-6\n"
+        "\tUse IPv6. By default IPv4 is used.\n"
+        "\n"
         "--bw-in n\n"
         "\tLimit incoming BitTorrent traffic to n kB/s.\n"
         "\tDefault is 0 which means unlimited.\n"
@@ -103,9 +127,8 @@ usage(void)
         "\tShow this text.\n"
         "\n"
         "--ip addr\n"
-        "\tMake other peers use the given address, instead of the one\n"
-        "\tthe tracker perceives as this peer's address, when contacting\n"
-        "\tthis peer.\n"
+        "\tLet the tracker distribute the given address instead of the one\n"
+        "\tit sees btpd connect from.\n"
         "\n"
         "--ipcprot mode\n"
         "\tSet the protection mode of the command socket.\n"
@@ -154,6 +177,7 @@ static struct option longopts[] = {
     { "ipcprot", required_argument,     &longval,       8 },
     { "empty-start", no_argument,       &longval,       9 },
     { "ip", required_argument,          &longval,       10 },
+    { "logmask", required_argument,     &longval,       11 },
     { "help",   no_argument,            &longval,       128 },
     { NULL,     0,                      NULL,           0 }
 };
@@ -162,12 +186,18 @@ int
 main(int argc, char **argv)
 {
     char *dir = NULL, *log = NULL;
-    int daemonize = 1;
+    int daemonize = 1, opt4 = 0, opt6 = 0;
 
     for (;;) {
-        switch (getopt_long(argc, argv, "d:p:", longopts, NULL)) {
+        switch (getopt_long(argc, argv, "46d:p:", longopts, NULL)) {
         case -1:
             goto args_done;
+        case '4':
+            opt4 = 1;
+            break;
+        case '6':
+            opt6 = 1;
+            break;
         case 'd':
             dir = optarg;
             break;
@@ -204,15 +234,10 @@ main(int argc, char **argv)
                 empty_start = 1;
                 break;
             case 10:
-                switch (inet_pton(AF_INET, optarg, &tr_ip_arg)) {
-                case 1:
-                    break;
-                case 0:
-                    errx(1, "You must specify a dotted IPv4 address.\n");
-                    break;
-                default:
-                    err(1, "inet_ntop %s", optarg);
-                }
+                tr_ip_arg = optarg;
+                break;
+            case 11:
+                btpd_logmask = atoi(optarg);
                 break;
             default:
                 usage();
@@ -227,24 +252,44 @@ args_done:
     argc -= optind;
     argv += optind;
 
+    if (opt6) {
+        net_ipv6 = 1;
+        if (!opt4)
+            net_ipv4 = 0;
+    }
+
     if (argc > 0)
         usage();
 
-    setup_daemon(daemonize, dir, log);
+    setup_daemon(daemonize, dir);
 
-    event_init();
-
-    if ((errno = evdns_resolv_conf_parse(DNS_OPTION_NAMESERVERS,
-             "/etc/resolv.conf")) != 0) {
-        btpd_log(BTPD_L_ERROR,
-            "failed to setup dns from /etc/resolv.conf (%d).\n", errno);
-    }
+    if (evloop_init() != 0)
+        btpd_err("Failed to initialize evloop (%s).\n", strerror(errno));
 
     btpd_init();
 
-    event_dispatch();
+    if (daemonize) {
+        if (freopen("/dev/null", "r", stdin) == NULL)
+            btpd_err("freopen of stdin failed (%s).\n", strerror(errno));
+        if (freopen(log == NULL ? "log" : log, "a", stderr) == NULL)
+            btpd_err("Couldn't open '%s' (%s).\n", log, strerror(errno));
+        if (dup2(fileno(stderr), fileno(stdout)) < 0)
+            btpd_err("dup2 failed (%s).\n", strerror(errno));
+        first_btpd_exit(0);
+    }
+    setlinebuf(stdout);
+    setlinebuf(stderr);
 
-    btpd_err("Unexpected exit from libevent.\n");
+    btpd_daemon_phase = 0;
+
+    if (!empty_start)
+        active_start();
+    else
+        active_clear();
+
+    evloop();
+
+    btpd_err("Exit from evloop with error (%s).\n", strerror(errno));
 
     return 1;
 }
