@@ -1,30 +1,21 @@
-#include <sys/types.h>
-#include <sys/uio.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-
-#include <sys/mman.h>
-#include <sys/wait.h>
-
-#include <assert.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <math.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-
 #include "btpd.h"
 
+#include <sys/uio.h>
+#include <netdb.h>
+
 static unsigned long m_bw_bytes_in;
 static unsigned long m_bw_bytes_out;
 
 static unsigned long m_rate_up;
 static unsigned long m_rate_dwn;
 
-static struct event m_net_incoming;
+struct net_listener {
+    int sd;
+    struct fdev ev;
+};
+
+static int m_nlisteners;
+static struct net_listener *m_net_listeners;
 
 unsigned net_npeers;
 
@@ -32,41 +23,51 @@ struct peer_tq net_bw_readq = BTPDQ_HEAD_INITIALIZER(net_bw_readq);
 struct peer_tq net_bw_writeq = BTPDQ_HEAD_INITIALIZER(net_bw_writeq);
 struct peer_tq net_unattached = BTPDQ_HEAD_INITIALIZER(net_unattached);
 
+void
+net_ban_peer(struct net *n, struct meta_peer *mp)
+{
+    if (mp->flags & PF_BANNED)
+        return;
+    mp_hold(mp); // Keep the meta peer alive
+    mp->flags |= PF_BANNED;
+    btpd_log(BTPD_L_BAD, "banned peer %p.\n", mp);
+}
+
 int
 net_torrent_has_peer(struct net *n, const uint8_t *id)
 {
-    int has = 0;
-    struct peer *p = BTPDQ_FIRST(&n->peers);
-    while (p != NULL) {
-        if (bcmp(p->id, id, 20) == 0) {
-            has = 1;
-            break;
-        }
-        p = BTPDQ_NEXT(p, p_entry);
-    }
-    return has;
+    return mptbl_find(n->mptbl, id) != NULL;
 }
 
 void
 net_create(struct torrent *tp)
 {
-    size_t field_size = ceil(tp->npieces / 8.0);
-    size_t mem = sizeof(*(tp->net)) + field_size +
-        tp->npieces * sizeof(*(tp->net->piece_count));
-
-    struct net *n = btpd_calloc(1, mem);
+    struct net *n = btpd_calloc(1, sizeof(*n));
     n->tp = tp;
     tp->net = n;
 
+    if ((n->mptbl = mptbl_create(3, btpd_id_eq, btpd_id_hash)) == NULL)
+        btpd_err("Out of memory.\n");
+
     BTPDQ_INIT(&n->getlst);
 
-    n->busy_field = (uint8_t *)(n + 1);
-    n->piece_count = (unsigned *)(n->busy_field + field_size);
+    n->busy_field = btpd_calloc(ceil(tp->npieces / 8.0), 1);
+    n->piece_count = btpd_calloc(tp->npieces, sizeof(*n->piece_count));
 }
 
 void
 net_kill(struct torrent *tp)
 {
+    struct htbl_iter it;
+    struct meta_peer *mp = mptbl_iter_first(tp->net->mptbl, &it);
+    while (mp != NULL) {
+        struct meta_peer *mps = mp;
+        mp = mptbl_iter_del(&it);
+        mp_kill(mps);
+    }
+    mptbl_free(tp->net->mptbl);
+    free(tp->net->piece_count);
+    free(tp->net->busy_field);
     free(tp->net);
     tp->net = NULL;
 }
@@ -169,7 +170,6 @@ net_write(struct peer *p, unsigned long wmax)
     nwritten = writev(p->sd, iov, niov);
     if (nwritten < 0) {
         if (errno == EAGAIN) {
-            btpd_ev_add(&p->out_ev, NULL);
             p->t_wantwrite = btpd_seconds;
             return 0;
         } else {
@@ -209,10 +209,10 @@ net_write(struct peer *p, unsigned long wmax)
             bcount = 0;
         }
     }
-    if (!BTPDQ_EMPTY(&p->outq)) {
-        btpd_ev_add(&p->out_ev, NULL);
+    if (!BTPDQ_EMPTY(&p->outq))
         p->t_wantwrite = btpd_seconds;
-    }
+    else
+        btpd_ev_disable(&p->ioev, EV_WRITE);
     p->t_lastwrite = btpd_seconds;
 
     return nwritten;
@@ -247,7 +247,7 @@ net_dispatch_msg(struct peer *p, const char *buf)
             res = 1;
         break;
     case MSG_REQUEST:
-        if ((p->flags & (PF_P_WANT|PF_I_CHOKE)) == PF_P_WANT) {
+        if ((p->mp->flags & (PF_P_WANT|PF_I_CHOKE)) == PF_P_WANT) {
             index = dec_be32(buf);
             begin = dec_be32(buf + 4);
             length = dec_be32(buf + 8);
@@ -322,7 +322,7 @@ net_state(struct peer *p, const char *buf)
         peer_set_in_state(p, SHAKE_INFO, 20);
         break;
     case SHAKE_INFO:
-        if (p->flags & PF_INCOMING) {
+        if (p->mp->flags & PF_INCOMING) {
             struct torrent *tp = torrent_by_hash(buf);
             if (tp == NULL || !net_active(tp))
                 goto bad;
@@ -336,7 +336,7 @@ net_state(struct peer *p, const char *buf)
         if ((net_torrent_has_peer(p->n, buf)
              || bcmp(buf, btpd_get_peer_id(), 20) == 0))
             goto bad;
-        bcopy(buf, p->id, 20);
+        bcopy(buf, p->mp->id, 20);
         peer_on_shake(p);
         peer_set_in_state(p, BTP_MSGSIZE, 4);
         break;
@@ -450,14 +450,13 @@ net_read(struct peer *p, unsigned long rmax)
     }
 
 out:
-    btpd_ev_add(&p->in_ev, NULL);
     return nread > 0 ? nread : 0;
 }
 
 int
-net_connect2(struct sockaddr *sa, socklen_t salen, int *sd)
+net_connect_addr(int family, struct sockaddr *sa, socklen_t salen, int *sd)
 {
-    if ((*sd = socket(PF_INET, SOCK_STREAM, 0)) == -1)
+    if ((*sd = socket(family, SOCK_STREAM, 0)) == -1)
         return errno;
 
     set_nonblocking(*sd);
@@ -473,7 +472,7 @@ net_connect2(struct sockaddr *sa, socklen_t salen, int *sd)
 }
 
 int
-net_connect(const char *ip, int port, int *sd)
+net_connect_name(const char *ip, int port, int *sd)
 {
     struct addrinfo hints, *res;
     char portstr[6];
@@ -483,13 +482,14 @@ net_connect(const char *ip, int port, int *sd)
     if (snprintf(portstr, sizeof(portstr), "%d", port) >= sizeof(portstr))
         return EINVAL;
     bzero(&hints, sizeof(hints));
-    hints.ai_family = AF_UNSPEC;
+    hints.ai_family = net_af_spec();
     hints.ai_flags = AI_NUMERICHOST;
     hints.ai_socktype = SOCK_STREAM;
     if (getaddrinfo(ip, portstr, &hints, &res) != 0)
         return errno;
 
-    int error = net_connect2(res->ai_addr, res->ai_addrlen, sd);
+    int error =
+        net_connect_addr(res->ai_family, res->ai_addr, res->ai_addrlen, sd);
     freeaddrinfo(res);
     return error;
 }
@@ -504,7 +504,7 @@ net_connection_cb(int sd, short type, void *arg)
         if (errno == EWOULDBLOCK || errno == ECONNABORTED)
             return;
         else
-            btpd_err("accept4: %s\n", strerror(errno));
+            btpd_err("accept: %s\n", strerror(errno));
     }
 
     if (set_nonblocking(nsd) != 0) {
@@ -572,13 +572,15 @@ net_bw_tick(void)
     if (net_bw_limit_in > 0) {
         while ((p = BTPDQ_FIRST(&net_bw_readq)) != NULL && m_bw_bytes_in > 0) {
             BTPDQ_REMOVE(&net_bw_readq, p, rq_entry);
-            p->flags &= ~PF_ON_READQ;
+            btpd_ev_enable(&p->ioev, EV_READ);
+            p->mp->flags &= ~PF_ON_READQ;
             m_bw_bytes_in -= net_read(p, m_bw_bytes_in);
         }
     } else {
         while ((p = BTPDQ_FIRST(&net_bw_readq)) != NULL) {
             BTPDQ_REMOVE(&net_bw_readq, p, rq_entry);
-            p->flags &= ~PF_ON_READQ;
+            btpd_ev_enable(&p->ioev, EV_READ);
+            p->mp->flags &= ~PF_ON_READQ;
             net_read(p, 0);
         }
     }
@@ -587,13 +589,15 @@ net_bw_tick(void)
         while (((p = BTPDQ_FIRST(&net_bw_writeq)) != NULL
                    && m_bw_bytes_out > 0)) {
             BTPDQ_REMOVE(&net_bw_writeq, p, wq_entry);
-            p->flags &= ~PF_ON_WRITEQ;
+            btpd_ev_enable(&p->ioev, EV_WRITE);
+            p->mp->flags &= ~PF_ON_WRITEQ;
             m_bw_bytes_out -=  net_write(p, m_bw_bytes_out);
         }
     } else {
         while ((p = BTPDQ_FIRST(&net_bw_writeq)) != NULL) {
             BTPDQ_REMOVE(&net_bw_writeq, p, wq_entry);
-            p->flags &= ~PF_ON_WRITEQ;
+            btpd_ev_enable(&p->ioev, EV_WRITE);
+            p->mp->flags &= ~PF_ON_WRITEQ;
             net_write(p, 0);
         }
     }
@@ -621,34 +625,69 @@ net_on_tick(void)
     net_bw_tick();
 }
 
-void
-net_read_cb(int sd, short type, void *arg)
+static void
+net_read_cb(struct peer *p)
 {
-    struct peer *p = (struct peer *)arg;
     if (net_bw_limit_in == 0)
         net_read(p, 0);
     else if (m_bw_bytes_in > 0)
         m_bw_bytes_in -= net_read(p, m_bw_bytes_in);
     else {
-        p->flags |= PF_ON_READQ;
+        btpd_ev_disable(&p->ioev, EV_READ);
+        p->mp->flags |= PF_ON_READQ;
         BTPDQ_INSERT_TAIL(&net_bw_readq, p, rq_entry);
     }
 }
 
-void
-net_write_cb(int sd, short type, void *arg)
+static void
+net_write_cb(struct peer *p)
 {
-    struct peer *p = (struct peer *)arg;
     if (net_bw_limit_out == 0)
         net_write(p, 0);
     else if (m_bw_bytes_out > 0)
         m_bw_bytes_out -= net_write(p, m_bw_bytes_out);
     else {
-        p->flags |= PF_ON_WRITEQ;
+        btpd_ev_disable(&p->ioev, EV_WRITE);
+        p->mp->flags |= PF_ON_WRITEQ;
         BTPDQ_INSERT_TAIL(&net_bw_writeq, p, wq_entry);
     }
 }
 
+void
+net_io_cb(int sd, short type, void *arg)
+{
+    switch (type) {
+    case EV_READ:
+        net_read_cb(arg);
+        break;
+    case EV_WRITE:
+        net_write_cb(arg);
+        break;
+    default:
+        abort();
+    }
+}
+
+int
+net_af_spec(void)
+{
+    if (net_ipv4 && net_ipv6)
+        return AF_UNSPEC;
+    else if (net_ipv4)
+        return AF_INET;
+    else
+        return AF_INET6;
+}
+
+void
+net_shutdown(void)
+{
+    for (int i = 0; i < m_nlisteners; i++) {
+        btpd_ev_del(&m_net_listeners[i].ev);
+        close(m_net_listeners[i].sd);
+    }
+}
+
 void
 net_init(void)
 {
@@ -656,27 +695,48 @@ net_init(void)
     m_bw_bytes_in = net_bw_limit_in;
 
     int safe_fds = getdtablesize() * 4 / 5;
-    if (strcmp(event_get_method(), "select") == 0)
-        safe_fds = min(safe_fds, FD_SETSIZE * 4 / 5);
     if (net_max_peers == 0 || net_max_peers > safe_fds)
         net_max_peers = safe_fds;
 
-    int sd;
-    int flag = 1;
-    struct sockaddr_in addr;
-    addr.sin_family = AF_INET;
-    addr.sin_addr.s_addr = htonl(INADDR_ANY);
-    addr.sin_port = htons(net_port);
-
-    if ((sd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
-        btpd_err("socket: %s\n", strerror(errno));
-    setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &flag, sizeof(flag));
-    if (bind(sd, (struct sockaddr *)&addr, sizeof(addr)) == -1)
-        btpd_err("bind: %s\n", strerror(errno));
-    listen(sd, 10);
-    set_nonblocking(sd);
-
-    event_set(&m_net_incoming, sd, EV_READ | EV_PERSIST,
-        net_connection_cb, NULL);
-    btpd_ev_add(&m_net_incoming, NULL);
+    int count = 0, flag = 1, found_ipv4 = 0, found_ipv6 = 0, sd;
+    char portstr[6];
+    struct addrinfo hints, *res, *ai;
+    bzero(&hints, sizeof(hints));
+    hints.ai_flags = AI_ADDRCONFIG | AI_PASSIVE;
+    hints.ai_family = net_af_spec();
+    hints.ai_socktype = SOCK_STREAM;
+    snprintf(portstr, sizeof(portstr), "%hu", net_port);
+    if ((errno = getaddrinfo(NULL, portstr, &hints, &res)) != 0)
+        btpd_err("getaddrinfo failed (%s).\n", gai_strerror(errno));
+    for (ai = res; ai != NULL; ai = ai->ai_next) {
+        count++;
+        if (ai->ai_family == AF_INET)
+            found_ipv4 = 1;
+        else
+            found_ipv6 = 1;
+    }
+    net_ipv4 = found_ipv4;
+    net_ipv6 = found_ipv6;
+    if (!net_ipv4 && !net_ipv6)
+        btpd_err("no usable address found. wrong use of -4/-6 perhaps.\n");
+    m_nlisteners = count;
+    m_net_listeners = btpd_calloc(count, sizeof(*m_net_listeners));
+    for (ai = res; ai != NULL; ai = ai->ai_next) {
+        count--;
+        if ((sd = socket(ai->ai_family, ai->ai_socktype, 0)) == -1)
+            btpd_err("failed to create socket (%s).\n", strerror(errno));
+        setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &flag, sizeof(flag));
+#ifdef IPV6_V6ONLY
+        if (ai->ai_family == AF_INET6)
+            setsockopt(sd, IPPROTO_IPV6, IPV6_V6ONLY, &flag, sizeof(flag));
+#endif
+        if (bind(sd, ai->ai_addr, ai->ai_addrlen) == -1)
+            btpd_err("bind failed (%s).\n", strerror(errno));
+        listen(sd, 10);
+        set_nonblocking(sd);
+        m_net_listeners[count].sd = sd;
+        btpd_ev_new(&m_net_listeners[count].ev, sd, EV_READ,
+            net_connection_cb, NULL);
+    }
+    freeaddrinfo(res);
 }
