@@ -29,12 +29,15 @@ void net_start(struct torrent *tp);
 void net_stop(struct torrent *tp);
 int net_active(struct torrent *tp);
 
+void net_ban_peer(struct net *n, struct meta_peer *mp);
 int net_torrent_has_peer(struct net *n, const uint8_t *id);
 
-void net_read_cb(int sd, short type, void *arg);
-void net_write_cb(int sd, short type, void *arg);
+void net_io_cb(int sd, short type, void *arg);
 
-int net_connect2(struct sockaddr *sa, socklen_t salen, int *sd);
-int net_connect(const char *ip, int port, int *sd);
+int net_connect_addr(int family, struct sockaddr *sa, socklen_t salen,
+    int *sd);
+int net_connect_name(const char *ip, int port, int *sd);
+
+int net_af_spec(void);
 
 #endif
