@@ -1,12 +1,7 @@
-#include <btpd.h>
-#include <netinet/in.h>
+#include "btpd.h"
 
 const char *btpd_dir;
-#ifdef DEBUG
-uint32_t btpd_logmask = BTPD_L_ALL;
-#else
 uint32_t btpd_logmask =  BTPD_L_BTPD | BTPD_L_ERROR;
-#endif
 int net_max_uploads = -2;
 unsigned net_max_peers;
 unsigned net_bw_limit_in;
@@ -15,4 +10,6 @@ int net_port = 6881;
 off_t cm_alloc_size = 2048 * 1024;
 int ipcprot = 0600;
 int empty_start = 0;
-uint32_t tr_ip_arg = INADDR_ANY;
+const char *tr_ip_arg;
+int net_ipv4 = 1;
+int net_ipv6 = 0;
