@@ -11,6 +11,7 @@ extern int net_port;
 extern off_t cm_alloc_size;
 extern int ipcprot;
 extern int empty_start;
-extern uint32_t tr_ip_arg;
+extern const char *tr_ip_arg;
+extern int net_ipv4, net_ipv6;
 
 #endif
