@@ -1,14 +1,9 @@
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/mman.h>
-
-#include <dirent.h>
-#include <fcntl.h>
-#include <string.h>
-#include <unistd.h>
-
 #include "btpd.h"
 
+#include <sys/mman.h>
+#include <dirent.h>
+#include <iobuf.h>
+
 HTBL_TYPE(numtbl, tlib, unsigned, num, nchain);
 HTBL_TYPE(hashtbl, tlib, uint8_t, hash, hchain);
 
@@ -35,6 +30,18 @@ tlib_by_hash(const uint8_t *hash)
     return hashtbl_find(m_hashtbl, hash);
 }
 
+struct tlib *
+tlib_iter_first(struct htbl_iter *it)
+{
+    return numtbl_iter_first(m_numtbl, it);
+}
+
+struct tlib *
+tlib_iter_next(struct htbl_iter *it)
+{
+    return numtbl_iter_next(it);
+}
+
 void
 tlib_kill(struct tlib *tl)
 {
@@ -67,12 +74,24 @@ int
 tlib_del(struct tlib *tl)
 {
     char relpath[RELPATH_SIZE];
-    char cmd[PATH_MAX];
-    assert(tl->tp == NULL);
-    snprintf(cmd, PATH_MAX, "rm -r torrents/%s",
-        bin2hex(tl->hash, relpath, 20));
-    system(cmd);
-    tlib_kill(tl);
+    char path[PATH_MAX];
+    DIR *dir;
+    struct dirent *de;
+    assert(tl->tp == NULL || tl->tp->state == T_GHOST);
+    snprintf(path, PATH_MAX, "torrents/%s", bin2hex(tl->hash, relpath, 20));
+    if ((dir = opendir(path)) != NULL) {
+        while ((de = readdir(dir)) != NULL) {
+            if (strcmp(".", de->d_name) == 0 || strcmp("..", de->d_name) == 0)
+                continue;
+            snprintf(path, PATH_MAX, "torrents/%s/%s", relpath, de->d_name);
+            remove(path);
+        }
+        closedir(dir);
+    }
+    snprintf(path, PATH_MAX, "torrents/%s", relpath);
+    remove(path);
+    if (tl->tp == NULL)
+        tlib_kill(tl);
     return 0;
 }
 
@@ -161,15 +180,15 @@ save_info(struct tlib *tl)
 {
     FILE *fp;
     char relpath[SHAHEXSIZE], path[PATH_MAX], wpath[PATH_MAX];
-    struct io_buffer iob = buf_init(1 << 10);
+    struct iobuf iob = iobuf_init(1 << 10);
 
-    buf_print(&iob,
+    iobuf_print(&iob,
         "d4:infod"
         "12:content havei%llde12:content sizei%llde"
         "3:dir%d:%s4:name%d:%s"
         "14:total downloadi%llde12:total uploadi%llde"
         "ee",
-        tl->content_have, tl->content_size,
+        (long long)tl->content_have, (long long)tl->content_size,
         (int)strlen(tl->dir), tl->dir, (int)strlen(tl->name), tl->name,
         tl->tot_down, tl->tot_up);
     if (iob.error)
@@ -182,7 +201,7 @@ save_info(struct tlib *tl)
     if ((fp = fopen(wpath, "w")) == NULL)
         btpd_err("failed to open '%s' (%s).\n", wpath, strerror(errno));
     dct_subst_save(fp, "de", iob.buf);
-    buf_free(&iob);
+    iobuf_free(&iob);
     if ((fflush(fp) == EOF || fsync(fileno(fp)) != 0
             || ferror(fp) || fclose(fp) != 0))
         btpd_err("failed to write '%s'.\n", wpath);
@@ -251,6 +270,19 @@ tlib_add(const uint8_t *hash, const char *mi, size_t mi_size,
     return tl;
 }
 
+struct tlib *
+tlib_readd(struct tlib *tl, const uint8_t *hash, const char *mi,
+    size_t mi_size, const char *content, char *name)
+{
+    struct tlib *tln;
+    struct torrent *tp = tl->tp;
+    tp->delete = 0;
+    tlib_kill(tl);
+    tln = tlib_add(hash, mi, mi_size, content, name);
+    tln->tp = tp;
+    return tln;
+}
+
 static int
 num_test(const void *k1, const void *k2)
 {
@@ -263,24 +295,6 @@ num_hash(const void *k)
     return *(const unsigned *)k;
 }
 
-static int
-id_test(const void *k1, const void *k2)
-{
-    return bcmp(k1, k2, 20) == 0;
-}
-
-static uint32_t
-id_hash(const void *k)
-{
-    return dec_be32(k + 16);
-}
-
-void
-tlib_put_all(struct tlib **v)
-{
-    hashtbl_tov(m_hashtbl, v);
-}
-
 void
 tlib_init(void)
 {
@@ -289,8 +303,8 @@ tlib_init(void)
     uint8_t hash[20];
     char file[PATH_MAX];
 
-    m_numtbl = numtbl_create(num_test, num_hash);
-    m_hashtbl = hashtbl_create(id_test, id_hash);
+    m_numtbl = numtbl_create(1, num_test, num_hash);
+    m_hashtbl = hashtbl_create(1, btpd_id_eq, btpd_id_hash);
     if (m_numtbl == NULL || m_hashtbl == NULL)
         btpd_err("Out of memory.\n");
 
