@@ -1,23 +1,10 @@
-#include <sys/types.h>
-#include <assert.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <math.h>
-#include <limits.h>
-#include <stdarg.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
+#include "btpd.h"
 
 #include <openssl/sha.h>
 
-#include "btpd.h"
-#include "tracker_req.h"
-#include "stream.h"
-
 #define SAVE_INTERVAL 300
 
+static unsigned m_nghosts;
 static unsigned m_ntorrents;
 static struct torrent_tq m_torrents = BTPDQ_HEAD_INITIALIZER(m_torrents);
 
@@ -36,6 +23,12 @@ torrent_count(void)
     return m_ntorrents;
 }
 
+unsigned
+torrent_ghosts(void)
+{
+    return m_nghosts;
+}
+
 struct torrent *
 torrent_by_num(unsigned num)
 {
@@ -85,12 +78,41 @@ torrent_block_size(struct torrent *tp, uint32_t piece, uint32_t nblocks,
     }
 }
 
+static void
+torrent_kill(struct torrent *tp)
+{
+    assert(m_ntorrents > 0);
+    assert(!(net_active(tp) || cm_active(tp)));
+    if (tp->state == T_GHOST)
+        m_nghosts--;
+    m_ntorrents--;
+    BTPDQ_REMOVE(&m_torrents, tp, entry);
+    if (tp->delete)
+        tlib_kill(tp->tl);
+    else
+        tp->tl->tp = NULL;
+    tr_kill(tp);
+    net_kill(tp);
+    cm_kill(tp);
+    mi_free_files(tp->nfiles, tp->files);
+    if (m_savetp == tp)
+        if ((m_savetp = BTPDQ_NEXT(tp, entry)) == NULL)
+            m_savetp = BTPDQ_FIRST(&m_torrents);
+    free(tp);
+}
+
 enum ipc_err
 torrent_start(struct tlib *tl)
 {
     struct torrent *tp;
     char *mi;
 
+    if (tl->tp != NULL) {
+        assert(torrent_startable(tl));
+        torrent_kill(tl->tp);
+        tl->tp = NULL;
+    }
+
     if (tl->dir == NULL)
         return IPC_EBADTENT;
 
@@ -110,46 +132,31 @@ torrent_start(struct tlib *tl)
         benc_dget_mem(benc_dget_dct(mi, "info"), "pieces", NULL) - mi;
 
     btpd_log(BTPD_L_BTPD, "Starting torrent '%s'.\n", torrent_name(tp));
-    if (tr_create(tp, mi) == 0) {
-        tl->tp = tp;
-        net_create(tp);
-        cm_create(tp, mi);
-        BTPDQ_INSERT_TAIL(&m_torrents, tp, entry);
-        m_ntorrents++;
-        cm_start(tp, 0);
-        free(mi);
-        if (m_ntorrents == 1) {
-            m_tsave = btpd_seconds + SAVE_INTERVAL;
-            m_savetp = tp;
-        }
-        return IPC_OK;
-    } else {
-        mi_free_files(tp->nfiles, tp->files);
-        free(tp);
-        free(mi);
-        return IPC_EBADTRACKER;
+    tr_create(tp, mi);
+    tl->tp = tp;
+    net_create(tp);
+    cm_create(tp, mi);
+    BTPDQ_INSERT_TAIL(&m_torrents, tp, entry);
+    m_ntorrents++;
+    cm_start(tp, 0);
+    free(mi);
+    if (m_ntorrents == 1) {
+        m_tsave = btpd_seconds + SAVE_INTERVAL;
+        m_savetp = tp;
     }
+    return IPC_OK;
 }
 
-static void
-torrent_kill(struct torrent *tp)
+static
+void become_ghost(struct torrent *tp)
 {
     btpd_log(BTPD_L_BTPD, "Stopped torrent '%s'.\n", torrent_name(tp));
-    assert(m_ntorrents > 0);
-    assert(!(tr_active(tp) || net_active(tp) || cm_active(tp)));
-    m_ntorrents--;
-    BTPDQ_REMOVE(&m_torrents, tp, entry);
-    tp->tl->tp = NULL;
+    tp->state = T_GHOST;
     if (tp->delete)
         tlib_del(tp->tl);
-    tr_kill(tp);
-    net_kill(tp);
-    cm_kill(tp);
-    mi_free_files(tp->nfiles, tp->files);
-    if (m_savetp == tp)
-        if ((m_savetp = BTPDQ_NEXT(tp, entry)) == NULL)
-            m_savetp = BTPDQ_FIRST(&m_torrents);
-    free(tp);
+    else
+        tlib_update_info(tp->tl, 0);
+    m_nghosts++;
 }
 
 void
@@ -168,12 +175,13 @@ torrent_stop(struct torrent *tp, int delete)
             tr_stop(tp);
         if (cm_active(tp))
             cm_stop(tp);
-        if (!delete)
-            tlib_update_info(tp->tl, 0);
+        if (!cm_active(tp)) {
+            become_ghost(tp);
+            if (!tr_active(tp))
+                torrent_kill(tp);
+        }
         break;
-    case T_STOPPING:
-        if (tr_active(tp))
-            tr_stop(tp);
+    default:
         break;
     }
 }
@@ -209,7 +217,11 @@ torrent_on_tick(struct torrent *tp)
         }
         break;
     case T_STOPPING:
-        if (!(cm_active(tp) || tr_active(tp)))
+        if (cm_active(tp))
+            break;
+        become_ghost(tp);
+    case T_GHOST:
+        if (!tr_active(tp))
             torrent_kill(tp);
         break;
     default:
@@ -235,3 +247,21 @@ torrent_on_tick_all(void)
         }
     }
 }
+
+int
+torrent_active(struct tlib *tl)
+{
+    return tl->tp != NULL && tl->tp->state != T_GHOST;
+}
+
+int
+torrent_startable(struct tlib *tl)
+{
+    return tl->tp == NULL || (tl->tp->state == T_GHOST && !tl->tp->delete);
+}
+
+int
+torrent_haunting(struct tlib *tl)
+{
+    return tl->tp != NULL && tl->tp->delete && tl->tp->state == T_GHOST;
+}
