@@ -8,7 +8,8 @@ enum torrent_state {
     T_STARTING,
     T_LEECH,
     T_SEED,
-    T_STOPPING
+    T_STOPPING,
+    T_GHOST
 };
 
 struct torrent {
@@ -18,7 +19,7 @@ struct torrent {
     int delete;
 
     struct content *cm;
-    struct tracker *tr;
+    struct trackers *tr;
     struct net *net;
 
     off_t total_length;
@@ -34,6 +35,7 @@ struct torrent {
 BTPDQ_HEAD(torrent_tq, torrent);
 
 unsigned torrent_count(void);
+unsigned torrent_ghosts(void);
 const struct torrent_tq *torrent_get_all(void);
 struct torrent *torrent_by_num(unsigned num);
 struct torrent *torrent_by_hash(const uint8_t *hash);
@@ -41,6 +43,10 @@ struct torrent *torrent_by_hash(const uint8_t *hash);
 enum ipc_err torrent_start(struct tlib *tl);
 void torrent_stop(struct torrent *tp, int delete);
 
+int torrent_active(struct tlib *tl);
+int torrent_haunting(struct tlib *tl);
+int torrent_startable(struct tlib *tl);
+
 off_t torrent_piece_size(struct torrent *tp, uint32_t piece);
 uint32_t torrent_piece_blocks(struct torrent *tp, uint32_t piece);
 uint32_t torrent_block_size(struct torrent *tp, uint32_t piece,
