@@ -1,7 +1,6 @@
 #ifndef BTCLI_H
 #define BTCLI_H
 
-#include <err.h>
 #include <errno.h>
 #include <getopt.h>
 #include <inttypes.h>
@@ -10,6 +9,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 #include <unistd.h>
 
 #include "btpd_if.h"
@@ -22,6 +22,8 @@
 extern const char *btpd_dir;
 extern struct ipc *ipc;
 
+__attribute__((noreturn))
+void diemsg(const char *fmt, ...);
 void btpd_connect(void);
 enum ipc_err handle_ipc_res(enum ipc_err err, const char *cmd,
     const char *target);
