@@ -11,8 +11,26 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 #include <unistd.h>
 
+#include "subr.h"
+
+void *
+memfind(const void *sub, size_t sublen, const void *mem, size_t memlen)
+{
+    size_t i, j;
+    const uint8_t *s = sub, *m = mem;
+    for (i = 0; i < memlen - sublen + 1; i++) {
+        for (j = 0; j < sublen; j++)
+            if (m[i+j] != s[j])
+                break;
+        if (j == sublen)
+            return (void *)(m + i);
+    }
+    return NULL;
+}
+
 void
 enc_be32(void *buf, uint32_t num)
 {
@@ -427,3 +445,21 @@ end:
     out[oi] = '\0';
     return 0;
 }
+
+#ifndef HAVE_ASPRINTF
+int
+asprintf(char **strp, const char *fmt, ...)
+{
+    int np;
+    va_list ap;
+    va_start(ap, fmt);
+    np = vsnprintf(NULL, 0, fmt, ap);
+    va_end(ap);
+    if ((*strp = malloc(np + 1)) == NULL)
+        return -1;
+    va_start(ap, fmt);
+    vsnprintf(*strp, np + 1, fmt, ap);
+    va_end(ap);
+    return np;
+}
+#endif
