@@ -1,14 +1,17 @@
 #ifndef BTPD_SUBR_H
 #define BTPD_SUBR_H
 
-#include <stdio.h>
 #include <stdarg.h>
+#include <stdio.h>
+#include <stdint.h>
 
 #define max(x, y) ((x) >= (y) ? (x) : (y))
 #define min(x, y) ((x) <= (y) ? (x) : (y))
 
 #define SHAHEXSIZE 41
 
+void *memfind(const void *sub, size_t sublen, const void *mem, size_t memlen);
+
 uint32_t dec_be32(const void *buf);
 uint64_t dec_be64(const void *buf);
 void enc_be32(void *buf, uint32_t num);
@@ -45,4 +48,9 @@ void *read_file(const char *path, void *buf, size_t *size);
 char *find_btpd_dir(void);
 int make_abs_path(const char *in, char *out);
 
+#ifndef HAVE_ASPRINTF
+__attribute__((format (printf, 2, 3)))
+int asprintf(char **strp, const char *fmt, ...);
+#endif
+
 #endif
